@@ -609,6 +609,16 @@ class ShapePolyTest(tf_test_util.JaxToTfTestCase):
         res_jax,
         jax2tf.convert(f, polymorphic_shapes=["(b, h)", "h"])(x, y))
 
+  def test_saved_model_shape_poly(self):
+    f_jax = jnp.sin
+    f_tf = jax2tf.convert(f_jax, polymorphic_shapes=["(b, ...)"])
+    x = np.array([0.7, 0.8], dtype=np.float32)
+    restored_f = tf_test_util.SaveAndLoadFunction(f_tf, [tf.TensorSpec([None], x.dtype)])
+    self.assertAllClose(f_jax(x), restored_f(x))
+    # Ensure that restored_f works at other batch size as well
+    y = np.concatenate([x, x])
+    self.assertAllClose(f_jax(y), restored_f(y))
+
   def test_readme_example(self):
     """Some of the examples from the README."""
     def image_mask_jax(images, mask):
@@ -874,6 +884,11 @@ _POLY_SHAPE_TEST_HARNESSES = [
                   [RandArg((3, 4), _f32)],
                   poly_axes=[0]),
 
+    _make_harness("add_transpose", "",
+                  jax.grad(lambda x: jnp.sum(jnp.sum(x, axis=0, keepdims=0) + x)),
+                  [RandArg((3, 4), _f32)],
+                   poly_axes=[0]),
+
     _make_harness("clamp", "",
                   lax.clamp,
                   [RandArg((3, 4, 5), _f32), RandArg((3, 4, 5), _f32),
@@ -939,16 +954,6 @@ _POLY_SHAPE_TEST_HARNESSES = [
                   [RandArg((3, 4), _f32), RandArg((4, 5), _f32)],
                   poly_axes=[1, 0]),
 
-    _make_harness("jnp_take", "",
-                  lambda a, i: jnp.take(a, i, axis=1),
-                  [RandArg((3, 4, 5), _f32), np.array([1, 2], np.int32)],
-                  poly_axes=[0, None]),
-
-    _make_harness("jnp_getitem", "",
-                  lambda a, i: a[i],
-                  [RandArg((3, 4), _f32), np.array([2, 2], np.int32)],
-                  poly_axes=[None, 0]),
-
     # TODO(necula): not supported yet
     # _make_harness("jnp_getitem", "",
     #               lambda a, i: a[i],
@@ -1067,14 +1072,24 @@ _POLY_SHAPE_TEST_HARNESSES = [
 ]
 
 for enable_xla in [False, True]:
-  _POLY_SHAPE_TEST_HARNESSES.append(
-      _make_harness(f"dynamic_slice_enablexla={enable_xla}", "",
+  _POLY_SHAPE_TEST_HARNESSES.extend([
+      _make_harness("dynamic_slice", f"enable_xla={enable_xla}",
                     # x:shape: (b, 4)
                     lambda x: lax.dynamic_slice(x, (0, 1), (x.shape[0], 2)),
                     [RandArg((3, 4), _f32)],
                     poly_axes=[0],
-                    enable_xla=enable_xla)
- )
+                    enable_xla=enable_xla),
+
+      _make_harness("jnp_take", f"enable_xla={enable_xla}",
+                    lambda a, i: jnp.take(a, i, axis=1),
+                    [RandArg((3, 4, 5), _f32), np.array([1, 2], np.int32)],
+                    poly_axes=[0, None], enable_xla=enable_xla),
+
+      _make_harness("jnp_getitem", f"enable_xla={enable_xla}",
+                    lambda a, i: a[i],
+                    [RandArg((3, 4), _f32), np.array([2, 2], np.int32)],
+                    poly_axes=[None, 0], enable_xla=enable_xla),
+ ])
 
 for reduce_op in [jnp.all, jnp.any, jnp.max, jnp.min, jnp.prod, jnp.sum]:
   _POLY_SHAPE_TEST_HARNESSES.append(
