@@ -15,10 +15,11 @@
 import contextlib
 import dataclasses
 import logging
+import os
 
 from typing import Any, Callable, List, Optional, Sequence
 
-
+from absl.testing import absltest
 import jax
 from jax import dtypes
 from jax import numpy as jnp
@@ -74,6 +75,28 @@ class OpMetadataGraph:
   source_line: str
 
 
+def SaveAndLoadModel(model: tf.Module,
+                     save_gradients=True) -> tf.Module:
+  # Roundtrip through saved model on disk.
+  model_dir = os.path.join(absltest.get_default_test_tmpdir(), str(id(model)))
+  tf.saved_model.save(
+      model, model_dir,
+      options=tf.saved_model.SaveOptions(experimental_custom_gradients=save_gradients))
+  restored_model = tf.saved_model.load(model_dir)
+  return restored_model
+
+def SaveAndLoadFunction(f_tf: Callable,
+                        input_signature: Sequence[tf.TensorSpec],
+                        save_gradients=True) -> Callable:
+  # Roundtrip through saved model on disk
+  model = tf.Module()
+  model.f = tf.function(f_tf,
+                        autograph=False,
+                        input_signature=input_signature)
+  restored = SaveAndLoadModel(model, save_gradients=save_gradients)
+  return restored.f
+
+
 class JaxToTfTestCase(jtu.JaxTestCase):
 
   def setUp(self):
@@ -344,7 +367,6 @@ class JaxToTfTestCase(jtu.JaxTestCase):
 
     return tree_util.tree_multimap(polymorphic_shape_to_tensorspec, polymorphic_shapes)
 
-
   def CheckOpMetadata(self, jax_fun, x,
                       expected: Sequence[OpMetadataGraph],
                       include_xla_op_metadata=True):
