@@ -25,7 +25,7 @@ from ..core import raise_to_shaped, get_aval, Literal, Jaxpr
 from ..api_util import flatten_fun_nokwargs
 from ..tree_util import tree_flatten, tree_unflatten, register_pytree_node
 from .._src.util import safe_map, safe_zip, split_list
-from .. import custom_derivatives
+from .._src import custom_derivatives
 
 map = safe_map
 zip = safe_zip
