@@ -41,7 +41,7 @@ _NOT_IMPLEMENTED = []
 def _init():
   import numpy as np
   from jax._src.numpy import lax_numpy
-  from jax import util
+  from jax._src import util
   # Builds a set of all unimplemented NumPy functions.
   for name, func in util.get_module_functions(np.fft).items():
     if name not in globals():
